@@ -112,6 +112,8 @@ struct NautilusBurnDrivePrivate {
 	gboolean                      monitor_enabled;
 	guint                         poll_id;
 
+	int                          *write_speeds;
+
 #ifdef USE_HAL
 	LibHalContext                *ctx;
 #endif
@@ -141,6 +143,24 @@ static GObjectClass *parent_class = NULL;
 
 G_DEFINE_TYPE(NautilusBurnDrive, nautilus_burn_drive, G_TYPE_OBJECT);
 
+/* fill in the write speeds as max_speed downto 1 if we
+ * are using a detection method that doesn't report all
+ * of the drive speeds
+ */
+static void
+fill_write_speeds (NautilusBurnDrive *drive)
+{
+	int max_speed;
+	int i;
+
+	max_speed = drive->max_speed_write;
+	drive->priv->write_speeds = g_new0 (int, max_speed + 1);
+
+	for (i = 0; i < max_speed; i++) {
+		drive->priv->write_speeds[i] = max_speed - i;
+	}
+}
+
 #ifdef __FreeBSD__
 
 #define get_ioctl_handle_fd(x) (((struct cam_device *)x)->fd)
@@ -186,6 +206,22 @@ close_ioctl_handle (gpointer handle)
 
 #endif
 
+static gboolean
+use_gnome_mount (void)
+{
+#ifndef USE_HAL
+	return FALSE;
+#endif
+
+	char *tmp = g_find_program_in_path ("gnome-mount");
+	if (tmp != NULL) {
+		g_free (tmp);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 #ifdef USE_HAL
 static LibHalContext *
 get_hal_context (void)
@@ -427,6 +463,30 @@ nautilus_burn_drive_get_max_speed_read (NautilusBurnDrive *drive)
 	return speed;
 }
 
+/**
+ * nautilus_burn_drive_get_max_write_speeds:
+ * @drive: #NautilusBurnDrive
+ *
+ * Get the list of supported write speeds
+ *
+ * Returns: The write speeds of the drive, in device units, as a
+ * zero-terminated array of integers.  The array is sorted in
+ * descending order and has no duplicate entries.
+ *
+ * Since: 2.14
+ **/
+const int *
+nautilus_burn_drive_get_write_speeds (NautilusBurnDrive *drive)
+{
+	const int *speeds;
+
+	g_return_val_if_fail (drive != NULL, NULL);
+
+	speeds = drive->priv->write_speeds;
+
+	return speeds;
+}
+
 /**
  * nautilus_burn_drive_get_name_for_display:
  * @drive: #NautilusBurnDrive
@@ -893,6 +953,57 @@ nautilus_burn_drive_get_media_type (NautilusBurnDrive *drive)
 }
 
 #ifdef USE_HAL
+
+static char *
+hal_drive_get_first_child_udi (NautilusBurnDrive *drive)
+{
+	LibHalContext        *ctx;
+	char                **device_names;
+	int                   num_devices;
+	DBusError             error;
+	char                 *udi;
+
+	udi = NULL;
+
+	if (drive->priv->udi == NULL) {
+		return NULL;
+	}
+
+	ctx = drive->priv->ctx;
+
+	if (ctx == NULL) {
+		return NULL;
+	}
+
+	num_devices = -1;
+	dbus_error_init (&error);
+	device_names = libhal_manager_find_device_string_match (ctx, 
+								"info.parent",
+								drive->priv->udi,
+								&num_devices,
+								&error);
+
+	if (dbus_error_is_set (&error)) {
+		g_warning ("%s\n", error.message);
+		dbus_error_free (&error);
+
+		goto done;
+	}
+
+	if (num_devices <= 0) {
+		g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
+
+		goto done;
+	}
+
+	udi = g_strdup (device_names [0]);
+
+ done:
+	libhal_free_string_array (device_names);
+
+	return udi;
+}
+
 static NautilusBurnMediaType
 nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 					     gboolean          *is_rewritable,
@@ -900,118 +1011,97 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 					     gboolean          *has_data,
 					     gboolean          *has_audio)
 {
+	LibHalContext        *ctx;
+	NautilusBurnMediaType type;
+	char                 *hal_type;
+	char                 *child_udi;
+
 	if (is_rewritable) *is_rewritable = FALSE;
 	if (is_blank) *is_blank = FALSE;
 	if (has_data) *has_data = FALSE;
 	if (has_audio) *has_audio = FALSE;
 
-	if (drive->priv != NULL && drive->priv->udi != NULL) {
-		LibHalContext        *ctx;
-		char                **device_names;
-		int                   num_devices = -1;
-		NautilusBurnMediaType type;
-		char                 *hal_type;
-		DBusError             error;
-		
-		ctx = drive->priv->ctx;
-
-		dbus_error_init (&error);
-		if (ctx != NULL) {
-			device_names = libhal_manager_find_device_string_match (ctx, 
-										"info.parent",
-										drive->priv->udi,
-										&num_devices,
-										&error);
-
-			if (dbus_error_is_set (&error)) {
-				g_warning ("%s\n", error.message);
-				dbus_error_free (&error);
-				libhal_free_string_array (device_names);
-
-				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
-			}
-
-			if (num_devices <= 0) {
-				g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
-				libhal_free_string_array (device_names);
-
-				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
-			}
-
-			/* just look at the first child */
-			if (libhal_device_get_property_bool (ctx, 
-							     device_names [0],
-							     "volume.is_mounted",
-							     NULL)) {
-				type = NAUTILUS_BURN_MEDIA_TYPE_BUSY;
-			} else {
-			
-				if (is_rewritable)
-					*is_rewritable = libhal_device_get_property_bool (ctx, 
-									 		  device_names [0],
-											  "volume.disc.is_rewritable",
-											  NULL);
-
-				if (is_blank)
-					*is_blank = libhal_device_get_property_bool (ctx, 
-										     device_names [0],
-										     "volume.disc.is_blank",
-										     NULL);
-
-				if (has_data)
-					*has_data = libhal_device_get_property_bool (ctx, 
-										     device_names [0],
-										     "volume.disc.has_data",
-										     NULL);
-
-				if (has_audio)
-					*has_audio = libhal_device_get_property_bool (ctx, 
-										      device_names [0],
-										      "volume.disc.has_audio",
-										      NULL);
-
-				type = NAUTILUS_BURN_MEDIA_TYPE_BUSY;
-				hal_type = libhal_device_get_property_string (ctx, 
-									      device_names [0],
-									      "volume.disc.type",
-									      NULL);
-
-				if (hal_type == NULL || strcmp (hal_type, "unknown") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
-				} else if (strcmp (hal_type, "cd_rom") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_CD;
-				} else if (strcmp (hal_type, "cd_r") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_CDR;
-				} else if (strcmp (hal_type, "cd_rw") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_CDRW;
-				} else if (strcmp (hal_type, "dvd_rom") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVD;
-				} else if (strcmp (hal_type, "dvd_r") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVDR;
-				} else if (strcmp (hal_type, "dvd_ram") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM;
-				} else if (strcmp (hal_type, "dvd_rw") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVDRW;
-				} else if (strcmp (hal_type, "dvd_plus_rw") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW;
-				} else if (strcmp (hal_type, "dvd_plus_r") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R;
-				} else if (strcmp (hal_type, "dvd_plus_r_dl") == 0) {
-					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL;
-				} else {
-					type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
-				}
-				
-				if (hal_type != NULL)
-					libhal_free_string (hal_type);
-			}
-
-			libhal_free_string_array (device_names);
-
-			return type;
-		}
+	if (drive->priv->udi == NULL) {
+		goto fallback;
 	}
 
+	ctx = drive->priv->ctx;
+
+	child_udi = hal_drive_get_first_child_udi (drive);
+	if (child_udi == NULL) {
+		goto fallback;
+	}
+
+	if (is_rewritable)
+		*is_rewritable = libhal_device_get_property_bool (ctx, 
+								  child_udi,
+								  "volume.disc.is_rewritable",
+								  NULL);
+
+	if (is_blank)
+		*is_blank = libhal_device_get_property_bool (ctx, 
+							     child_udi,
+							     "volume.disc.is_blank",
+							     NULL);
+
+	if (has_data)
+		*has_data = libhal_device_get_property_bool (ctx, 
+							     child_udi,
+							     "volume.disc.has_data",
+							     NULL);
+
+	if (has_audio)
+		*has_audio = libhal_device_get_property_bool (ctx, 
+							      child_udi,
+							      "volume.disc.has_audio",
+							      NULL);
+
+	type = NAUTILUS_BURN_MEDIA_TYPE_BUSY;
+	hal_type = libhal_device_get_property_string (ctx, 
+						      child_udi,
+						      "volume.disc.type",
+						      NULL);
+
+	if (hal_type == NULL || strcmp (hal_type, "unknown") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+	} else if (strcmp (hal_type, "cd_rom") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_CD;
+	} else if (strcmp (hal_type, "cd_r") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_CDR;
+	} else if (strcmp (hal_type, "cd_rw") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_CDRW;
+	} else if (strcmp (hal_type, "dvd_rom") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVD;
+	} else if (strcmp (hal_type, "dvd_r") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVDR;
+	} else if (strcmp (hal_type, "dvd_ram") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM;
+	} else if (strcmp (hal_type, "dvd_rw") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVDRW;
+	} else if (strcmp (hal_type, "dvd_plus_rw") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW;
+	} else if (strcmp (hal_type, "dvd_plus_r") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R;
+	} else if (strcmp (hal_type, "dvd_plus_r_dl") == 0) {
+		type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL;
+	} else {
+		type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+	}
+				
+	if (hal_type != NULL) {
+		libhal_free_string (hal_type);
+	}
+
+	g_free (child_udi);
+
+	/* if the type has been determined using HAL, return it.
+	   Otherwise, fall through and try the non-HAL approach. */
+	if (type != NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN
+	    && type != NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+		return type;
+	}
+
+ fallback:
 	return nautilus_burn_drive_get_media_type_from_path_full (drive->device,
 								  is_rewritable,
 								  is_blank,
@@ -1198,6 +1288,55 @@ try_media_size_from_type (NautilusBurnDrive *drive)
 	return size;
 }
 
+#ifdef USE_HAL
+static gint64
+nautilus_burn_drive_hal_get_media_size (NautilusBurnDrive *drive)
+{
+	gint64         size;
+	guint64        res;
+	LibHalContext *ctx;
+	char          *child_udi;
+	DBusError      error;
+
+	size = -1;
+
+	if (drive->priv->udi == NULL) {
+		goto done;
+	}
+		
+	ctx = drive->priv->ctx;
+
+	if (ctx == NULL) {
+		goto done;
+	}
+
+	child_udi = hal_drive_get_first_child_udi (drive);
+	if (child_udi == NULL) {
+		goto done;
+	}
+
+	dbus_error_init (&error);
+	res = libhal_device_get_property_uint64 (ctx, 
+						 child_udi,
+						 "volume.disc.capacity",
+						 &error);
+
+	g_free (child_udi);
+
+	if (dbus_error_is_set (&error)) {
+		g_warning ("%s\n", error.message);
+		dbus_error_free (&error);
+				
+		goto done;
+	}
+
+	size = (gint64) res;
+
+ done:
+	return size;
+}
+#endif
+
 /**
  * nautilus_burn_drive_get_media_size:
  * @drive: #NautilusBurnDrive
@@ -1219,9 +1358,12 @@ nautilus_burn_drive_get_media_size (NautilusBurnDrive *drive)
 	g_return_val_if_fail (drive != NULL, NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN);
 
 #ifdef USE_HAL
-	size = nautilus_burn_drive_get_media_size_from_path (drive->device);
+	size = nautilus_burn_drive_hal_get_media_size (drive);
 	if (size < 0) {
-		size = try_media_size_from_type (drive);
+		size = nautilus_burn_drive_get_media_size_from_path (drive->device);
+		if (size < 0) {
+			size = try_media_size_from_type (drive);
+		}
 	}
 #else
 	size = nautilus_burn_drive_get_media_size_from_path (drive->device);
@@ -1230,27 +1372,91 @@ nautilus_burn_drive_get_media_size (NautilusBurnDrive *drive)
 	return size;
 }
 
+#ifdef USE_HAL
+static char *
+nautilus_burn_drive_hal_get_media_label (NautilusBurnDrive *drive)
+{
+	char          *label;
+	LibHalContext *ctx;
+	char          *child_udi;
+	DBusError      error;
+
+	g_return_val_if_fail (drive != NULL, NULL);
+
+	label = NULL;
+
+	if (drive->priv->udi == NULL) {
+		goto done;
+	}
+		
+	ctx = drive->priv->ctx;
+
+	if (ctx == NULL) {
+		goto done;
+	}
+
+	child_udi = hal_drive_get_first_child_udi (drive);
+	if (child_udi == NULL) {
+		goto done;
+	}
+
+	dbus_error_init (&error);
+	label = libhal_device_get_property_string (ctx, 
+						   child_udi,
+						   "volume.label",
+						   &error);
+	if (dbus_error_is_set (&error)) {
+		g_warning ("%s\n", error.message);
+		dbus_error_free (&error);
+	}
+
+	g_free (child_udi);
+ done:
+	return label;
+}
+#endif
+
+/**
+ * nautilus_burn_drive_get_media_label:
+ * @drive: #NautilusBurnDrive
+ *
+ * Determine the label of the media in the specified drive.
+ *
+ * Return value: The label of the media.
+ *
+ * Since: 2.14
+ **/
+char *
+nautilus_burn_drive_get_media_label (NautilusBurnDrive *drive)
+{
+	char *label;
+
+	g_return_val_if_fail (drive != NULL, NULL);
+
+	label = NULL;
+#ifdef USE_HAL
+	label = nautilus_burn_drive_hal_get_media_label (drive);
+#else
+	g_warning ("nautilus_burn_drive_get_media_label: not implemented without HAL");
+#endif
+
+	return label;
+}
+
 typedef struct {
 	gboolean    timeout;
 	gboolean    unmount_ok;
 	guint       timeout_tag;
 	GMainLoop  *loop;
-	char       *device;
-	const char *command;
+	GPtrArray  *argv;
 } UnmountData;
 
-static const char *umount_known_locations [] = {
-	"/sbin/umount",
-	"/bin/umount",
-	"/usr/sbin/umount",
-	"/usr/bin/umount",
-	NULL
-};
-
 static void
 free_unmount_data (UnmountData *unmount_data)
 {
-	g_free (unmount_data->device);
+	g_ptr_array_foreach (unmount_data->argv, (GFunc)g_free, NULL);
+	g_ptr_array_free (unmount_data->argv, TRUE);
+
 	g_free (unmount_data);
 }
 
@@ -1298,17 +1504,56 @@ unmount_timeout (gpointer data)
 	return FALSE;
 }
 
-/* Returns the full path to the queried command */
-static const char *
-find_command (const char **known_locations)
-{
-	int i;
+static const char *umount_known_locations [] = {
+	"/sbin/umount",
+	"/bin/umount",
+	"/usr/sbin/umount",
+	"/usr/bin/umount",
+	NULL
+};
 
-	for (i = 0; known_locations [i]; i++){
-		if (g_file_test (known_locations [i], G_FILE_TEST_EXISTS))
-			return known_locations [i];
+/* Returns the full command */
+static GPtrArray *
+create_command (const char *device)
+{
+	GPtrArray *argv;
+
+	argv = g_ptr_array_new ();
+
+	if (use_gnome_mount ()) {
+		char *str;
+
+		str = g_strdup ("gnome-mount");
+		g_ptr_array_add (argv, str);
+		str = g_strdup_printf ("--device=%s", device);
+		g_ptr_array_add (argv, str);
+		str = g_strdup ("--unmount");
+		g_ptr_array_add (argv, str);
+		str = g_strdup ("--no-ui");
+		g_ptr_array_add (argv, str);
+	} else {
+		char *str;
+		int   i;
+
+		str = NULL;
+		for (i = 0; umount_known_locations [i]; i++){
+			if (g_file_test (umount_known_locations [i], G_FILE_TEST_EXISTS)) {
+				str = g_strdup (umount_known_locations [i]);
+				break;
+			}
+		}
+
+		/* no path then try just command */
+		if (str == NULL) {
+			str = g_strdup ("umount");
+		}
+
+		g_ptr_array_add (argv, str);
+		str = g_strdup_printf ("%s", device);
+		g_ptr_array_add (argv, str);
 	}
-	return NULL;
+
+	return argv;
 }
 
 static void *
@@ -1316,8 +1561,6 @@ unmount_thread_start (void *arg)
 {
 	UnmountData *data;
 	gint         exit_status;
-	char        *argv [5];
-	int          i;
 	char        *envp [] = {
 		"LC_ALL=C",
 		NULL
@@ -1327,13 +1570,8 @@ unmount_thread_start (void *arg)
 
 	data->unmount_ok = TRUE;
 	
-	i = 0;
-	argv [i++] = (char *)data->command;
-	argv [i++] = data->device;
-	argv [i++] = NULL;
-	
 	if (g_spawn_sync (NULL,
-			  argv,
+			  (char **)data->argv->pdata,
 			  envp,
 			  G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
 			  NULL, NULL,
@@ -1390,8 +1628,8 @@ nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 	data->timeout_tag = g_timeout_add (5 * 1000,
 					   unmount_timeout,
 					   data);
-	data->command = find_command (umount_known_locations);
-	data->device = g_strdup (drive->device);
+	data->argv = create_command (drive->device);
+
 	g_thread_create (unmount_thread_start, data, FALSE, NULL);
 	
 	GDK_THREADS_LEAVE ();
@@ -1432,7 +1670,12 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 	if (drive->device == NULL)
 		return FALSE;
 	
-	cmd = g_strdup_printf ("eject %s", drive->device);
+	if (use_gnome_mount ()) {
+		cmd = g_strdup_printf ("gnome-mount --eject --no-ui --device=%s", drive->device);
+	} else {
+		cmd = g_strdup_printf ("eject %s", drive->device);
+	}
+
 	res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
 	g_free (cmd);
 
@@ -1571,6 +1814,41 @@ drive_hal_init (NautilusBurnDrive *drive)
 	drive->priv->ctx = ctx;
 }
 
+static int *
+hal_parse_write_speeds (char **strlist)
+{
+	char *end;
+	int  *write_speeds;
+	int   fields = 1;
+	int   i;
+
+	if (strlist == NULL)
+		return NULL;
+
+	fields = libhal_string_array_length (strlist);
+
+	if (fields == 0)
+		return NULL;
+
+	write_speeds = g_new0 (int, fields + 1);
+
+	for (i = 0; i < fields; i++) {
+		write_speeds[i] = strtol (strlist[i], &end, 10);
+
+		if (write_speeds[i] < 0
+		    || write_speeds[i] > 65535
+		    || *end != '\0') {
+
+			g_free (write_speeds);
+			return NULL;
+		}
+
+		write_speeds[i] /= CD_ROM_SPEED;
+	}
+
+	return write_speeds;
+}
+
 #define LIBHAL_PROP_EXTRACT_BEGIN if (FALSE)
 #define LIBHAL_PROP_EXTRACT_END ;
 #define LIBHAL_PROP_EXTRACT_INT(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it)
@@ -1578,7 +1856,7 @@ drive_hal_init (NautilusBurnDrive *drive)
 #define LIBHAL_PROP_EXTRACT_STRING(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRING) _where_ = (libhal_psi_get_string (&it) != NULL && strlen (libhal_psi_get_string (&it)) > 0) ? strdup (libhal_psi_get_string (&it)) : NULL
 #define LIBHAL_PROP_EXTRACT_BOOL(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ = libhal_psi_get_bool (&it)
 #define LIBHAL_PROP_EXTRACT_BOOL_BITFIELD(_property_, _where_, _field_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ |= libhal_psi_get_bool (&it) ? _field_ : 0
-#define LIBHAL_PROP_EXTRACT_STRLIST(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRLIST) _where_ = my_strvdup (libhal_psi_get_strlist (&it))
+#define LIBHAL_PROP_EXTRACT_STRLIST(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRLIST) _where_ = libhal_psi_get_strlist (&it)
 
 static NautilusBurnDrive *
 hal_drive_from_udi (LibHalContext *ctx,
@@ -1588,6 +1866,7 @@ hal_drive_from_udi (LibHalContext *ctx,
 	LibHalPropertySetIterator it;
 	DBusError                 error;
 	NautilusBurnDrive        *drive;
+	char                    **write_speeds = NULL;
 
 	LIBHAL_CHECK_LIBHALCONTEXT (ctx, FALSE);
 
@@ -1614,12 +1893,12 @@ hal_drive_from_udi (LibHalContext *ctx,
 
 		LIBHAL_PROP_EXTRACT_BEGIN;
 
-		LIBHAL_PROP_EXTRACT_STRING  ("block.device",              drive->device);
-		LIBHAL_PROP_EXTRACT_STRING  ("storage.model",             drive->display_name);
-
-		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.read_speed",  drive->max_speed_read,  CD_ROM_SPEED);
-		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.write_speed", drive->max_speed_write, CD_ROM_SPEED);
+		LIBHAL_PROP_EXTRACT_STRING  ("block.device",               drive->device);
+		LIBHAL_PROP_EXTRACT_STRING  ("storage.model",              drive->display_name);
 
+		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.read_speed",   drive->max_speed_read,  CD_ROM_SPEED);
+		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.write_speed",  drive->max_speed_write, CD_ROM_SPEED);
+		LIBHAL_PROP_EXTRACT_STRLIST ("storage.cdrom.write_speeds", write_speeds);
 
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.cdr",        drive->type, NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.cdrw",       drive->type, NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER);
@@ -1634,14 +1913,19 @@ hal_drive_from_udi (LibHalContext *ctx,
 
 		LIBHAL_PROP_EXTRACT_END;
 	}
-
-	libhal_free_property_set (pset);
-
 	drive->priv->udi = g_strdup (udi);
 	drive->cdrecord_id = g_strdup (drive->device);
+	drive->priv->write_speeds = hal_parse_write_speeds (write_speeds);
+	/* we do not own write_speeds so do not free it. */
+	if (drive->priv->write_speeds == NULL) {
+		fill_write_speeds (drive);
+	}
 
-	if (! drive->display_name)
+	if (! drive->display_name) {
 		drive->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->device);
+	}
+
+	libhal_free_property_set (pset);
 
 	return drive;
 }
@@ -2013,6 +2297,7 @@ add_linux_cd_recorder (GList             *drives,
 		/* Have a wild guess, the drive should actually correct us */
 		drive->max_speed_write = drive_s->speed;
 	}
+	fill_write_speeds (drive);
 
 	drive->device = g_strdup_printf ("/dev/%s", drive_s->device);
 	drive->max_speed_read = drive_s->speed;
@@ -2052,6 +2337,7 @@ add_linux_cd_drive (GList             *drives,
 	drive->display_name = g_strdup (drive_s->display_name);
 	drive->device = g_strdup_printf ("/dev/%s", drive_s->device);
 	drive->max_speed_write = 0; /* Can't write */
+	fill_write_speeds (drive);
 	drive->max_speed_read = drive_s->speed;
 	if (drive_s->can_read_dvd) {
 		drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
@@ -2350,6 +2636,7 @@ freebsd_scan (gboolean recorder_only)
 			if (drive->max_speed_write == -1) {
 			    	drive->max_speed_write = speed;
 			}
+			fill_write_speeds (drive);
 
 			if (drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE) {
 				add_dvd_plus (drive);
@@ -2453,6 +2740,7 @@ cdrecord_scan (gboolean recorder_only)
 			if (drive->max_speed_write == -1) {
 			    	drive->max_speed_write = DEFAULT_SPEED;
 			}
+			fill_write_speeds (drive);
 
 			if (drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE) {
 				add_dvd_plus (drive);
@@ -2487,6 +2775,7 @@ nautilus_burn_drive_get_file_image (void)
 	drive->display_name = g_strdup (_("File image"));
 	drive->max_speed_read = 0;
 	drive->max_speed_write = 0;
+	fill_write_speeds (drive);
 	drive->type = NAUTILUS_BURN_DRIVE_TYPE_FILE;
 
 	return drive;
@@ -2713,6 +3002,7 @@ nautilus_burn_drive_finalize (GObject *object)
 	g_free (drive->priv->udi);
 	g_free (drive->priv->monitor_udi);
 
+	g_free (drive->priv->write_speeds);
 	g_free (drive->display_name);
 	g_free (drive->cdrecord_id);
 	g_free (drive->device);
@@ -2731,6 +3021,7 @@ nautilus_burn_drive_init (NautilusBurnDrive *drive)
 	drive->display_name    = NULL;
 	drive->max_speed_write = 0;
 	drive->max_speed_read  = 0;
+	drive->priv->write_speeds = NULL;
 	drive->cdrecord_id     = NULL;
 	drive->device          = NULL;
 
@@ -2946,31 +3237,16 @@ set_monitor_enabled_hal (NautilusBurnDrive *drive,
 	drive->priv->monitor_udi = NULL;
 
 	if (enabled) {
-		char    **device_names;
-		int       num_devices = -1;
-		DBusError error;
+		char *child_udi;
 
 		libhal_ctx_set_user_data (drive->priv->ctx, drive);
 		libhal_ctx_set_device_added (drive->priv->ctx, hal_device_added);
 		libhal_ctx_set_device_removed (drive->priv->ctx, hal_device_removed);
 
-		dbus_error_init (&error);
-		device_names = libhal_manager_find_device_string_match (drive->priv->ctx, 
-									"info.parent",
-									drive->priv->udi,
-									&num_devices,
-									&error);
-		
-		if (dbus_error_is_set (&error)) {
-			g_warning ("%s\n", error.message);
-			dbus_error_free (&error);
+		child_udi = hal_drive_get_first_child_udi (drive);
+		if (child_udi != NULL) {
+			drive->priv->monitor_udi = child_udi;
 		}
-
-		if (num_devices > 0) {
-			drive->priv->monitor_udi = g_strdup (device_names [0]);
-		}
-
-		libhal_free_string_array (device_names);
 	} else {
 		libhal_ctx_set_user_data (drive->priv->ctx, NULL);
 		libhal_ctx_set_device_added (drive->priv->ctx, NULL);
