@@ -31,6 +31,8 @@
 #include <glib-object.h>
 #include <glib.h>
 
+#include "nautilus-burn-features.h"
+
 G_BEGIN_DECLS
 
 typedef enum {
@@ -133,9 +135,9 @@ gboolean              nautilus_burn_drive_can_rewrite              (NautilusBurn
 
 int                   nautilus_burn_drive_get_max_speed_write      (NautilusBurnDrive       *drive);
 int                   nautilus_burn_drive_get_max_speed_read       (NautilusBurnDrive       *drive);
+const int *           nautilus_burn_drive_get_write_speeds         (NautilusBurnDrive       *drive);
 
-
-/* Media type handling */
+/* Media handling */
 
 NautilusBurnMediaType nautilus_burn_drive_get_media_type           (NautilusBurnDrive       *drive);
 
@@ -145,7 +147,7 @@ NautilusBurnMediaType nautilus_burn_drive_get_media_type_full      (NautilusBurn
 								    gboolean                *has_data,
 								    gboolean                *has_audio);
 gint64                nautilus_burn_drive_get_media_size           (NautilusBurnDrive       *drive);
-
+char *                nautilus_burn_drive_get_media_label          (NautilusBurnDrive       *drive);
 
 const char *          nautilus_burn_drive_media_type_get_string    (NautilusBurnMediaType    type);
 char *                nautilus_burn_drive_get_supported_media_string (NautilusBurnDrive       *drive,
