@@ -3,7 +3,7 @@
  * nautilus-cd-burner.c: easy to use cd burner software
  * 
  * Copyright (C) 2002-2004 Red Hat, Inc.
- * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2005-2006 William Jon McCann <mccann@jhu.edu>
  *   
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,8 @@
  *          William Jon McCann <mccann@jhu.edu>
  */
 
+#include "config.h"
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,10 +36,9 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkfilesel.h>
-#include <gtk/gtkseparatormenuitem.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
+
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <gconf/gconf-client.h>
@@ -529,7 +530,9 @@ ask_for_media (NautilusBurnDrive *drive,
 {
 	GtkWidget  *dialog;
 	GtkWidget  *parent;
+	GtkWidget  *button;
 	gboolean    can_rewrite;
+	gboolean    show_eject;
 	char       *msg;
 	char       *type_string;
 	const char *title;
@@ -541,6 +544,7 @@ ask_for_media (NautilusBurnDrive *drive,
 	type_string = nautilus_burn_drive_get_supported_media_string (drive, TRUE);
 
 	can_rewrite = nautilus_burn_drive_can_rewrite (drive);
+	show_eject = FALSE;
 
 	if (busy_cd) {
 		msg = g_strdup (_("Please make sure another application is not using the drive."));
@@ -556,11 +560,13 @@ ask_for_media (NautilusBurnDrive *drive,
 				       type_string);
 		title = N_("Insert a blank disc");
 	} else if (can_rewrite) {
+		show_eject = TRUE;
 		msg = g_strdup_printf (_("Please replace the disc in the drive with a supported disc with at least %d MiB free.  The following disc types are supported:\n%s"),
 				       mibs,
 				       type_string);
 		title = N_("Reload a rewritable or blank disc");
 	} else {
+		show_eject = TRUE;
 		msg = g_strdup_printf (_("Please replace the disc in the drive with a supported disc with at least %d MiB free.  The following disc types are supported:\n%s"),
 				       mibs,
 				       type_string);
@@ -576,13 +582,26 @@ ask_for_media (NautilusBurnDrive *drive,
 	dialog = ncb_hig_dialog (GTK_MESSAGE_INFO,
 				 _(title), _(msg), GTK_WINDOW (parent));
 
+	if (show_eject) {
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						_("_Eject"),
+						GTK_RESPONSE_NO);
+	}
+
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_OK);
 
+ retry:
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if (res == GTK_RESPONSE_NO) {
+		nautilus_burn_drive_eject (drive);
+		goto retry;
+	}
+
 	gtk_widget_destroy (dialog);
 
 	g_free (msg);
@@ -858,6 +877,19 @@ setup_cancel_button (void)
 	/* FIXME: disconnect signal handlers? */
 }
 
+static void
+eject_disc_cb (GtkWidget *widget,
+	       gpointer   user_data)
+	       
+{
+	NautilusBurnerWriteData *data;
+
+	data = nautilus_burn_progress_get_write_data ();
+	if (data != NULL && data->drive != NULL) {
+		nautilus_burn_drive_eject ((NautilusBurnDrive *)data->drive);
+	}
+}
+
 static void
 write_another_disc_cb (GtkWidget *widget,
 		       gpointer   user_data)
@@ -868,7 +900,9 @@ write_another_disc_cb (GtkWidget *widget,
 	setup_cancel_button ();
 
 	data = nautilus_burn_progress_get_write_data ();
-	nautilus_burn_drive_eject ((NautilusBurnDrive *)data->drive);
+	if (data != NULL && data->drive != NULL) {
+		nautilus_burn_drive_eject ((NautilusBurnDrive *)data->drive);
+	}
 
 	res = nautilus_burn_progress_write_disc ();
 
@@ -895,7 +929,7 @@ setup_close_button (void)
 	g_signal_connect (close_button, "clicked", G_CALLBACK (gtk_main_quit), NULL);
 
 	data = nautilus_burn_progress_get_write_data ();
-	if (data) {
+	if (data != NULL) {
 		GtkWidget *write_another_button;
 		write_another_button = glade_xml_get_widget (xml, "write_another_button");
 		gtk_widget_show (write_another_button);
@@ -903,6 +937,15 @@ setup_close_button (void)
 		g_signal_handlers_disconnect_by_func (write_another_button, write_another_disc_cb, NULL);
 		g_signal_connect (write_another_button, "clicked", G_CALLBACK (write_another_disc_cb), NULL);
 	}
+
+	if (data != NULL) {
+		GtkWidget *button;
+		button = glade_xml_get_widget (xml, "eject_button");
+		gtk_widget_show (button);
+
+		g_signal_handlers_disconnect_by_func (button, eject_disc_cb, NULL);
+		g_signal_connect (button, "clicked", G_CALLBACK (eject_disc_cb), NULL);
+	}
 }
 
 static gboolean 
@@ -937,7 +980,7 @@ overwrite_existing_file (GtkWindow  *filesel,
 }
 
 static char *
-select_iso_filename (void)
+select_iso_filename (const char *default_name)
 {
 	GtkWidget *chooser;
 	char      *filename;
@@ -951,7 +994,7 @@ select_iso_filename (void)
 							    GTK_STOCK_OK, GTK_RESPONSE_OK,
 							    NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser),
-					   "image.iso");
+					   default_name);
  reselect:
 	filename = NULL;
 	response = gtk_dialog_run (GTK_DIALOG (chooser));
@@ -1104,12 +1147,24 @@ request_media (NautilusBurnDrive *drive,
 }
 
 static char *
-get_output_filename (void)
+get_output_filename (const char *disc_name)
 {
-	char *filename = NULL;
+	char *filename         = NULL;
+	char *default_filename = NULL;
+
+	if (disc_name) {
+		default_filename = g_strdup_printf ("%s.iso", disc_name);
+	}
+
+	if (! default_filename) {
+		/* Translators: this is the filename of the image */
+		default_filename = g_strdup (_("image.iso"));
+	}
 
 	/* Run the file selector and get the iso file name selected */
-	filename = select_iso_filename ();
+	filename = select_iso_filename (default_filename);
+	g_free (default_filename);
+
 	if (! filename)
 		return NULL;
 
@@ -1347,10 +1402,10 @@ update_progress_dialog_disc_type (NautilusBurnDrive        *drive,
 	heading = glade_xml_get_widget (xml, "info_title_label");
 	dialog  = glade_xml_get_widget (xml, "progress_window");
 
-	gtk_label_set_text (GTK_LABEL (label), label_text);
-	gtk_window_set_title (GTK_WINDOW (dialog), title_text);
+	gtk_label_set_text (GTK_LABEL (label), _(label_text));
+	gtk_window_set_title (GTK_WINDOW (dialog), _(title_text));
 
-	heading_markup = g_strdup_printf ("<big><b>%s</b></big>", heading_text);
+	heading_markup = g_strdup_printf ("<big><b>%s</b></big>", _(heading_text));
 	gtk_label_set_markup (GTK_LABEL (heading), heading_markup);
 	g_free (heading_markup);
 }
@@ -1388,7 +1443,14 @@ burn_cd (NautilusBurnDrive        *drive,
 	filename = NULL;
 
 	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
-		filename = get_output_filename ();
+		char *disc_name = NULL;
+
+		if (source_drive)
+			disc_name = nautilus_burn_drive_get_media_label (source_drive);
+
+		filename = get_output_filename (disc_name);
+		g_free (disc_name);
+
 		if (! filename)
 			return NAUTILUS_BURN_RECORDER_RESULT_CANCEL;
 	} else {
@@ -1665,10 +1727,14 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget               *combobox;
 	GConfClient             *gc;
 	char                    *name;
-	int                      i, default_speed;
+	int                      i;
+	int                      default_speed;
+	int                      default_speed_index;
 	NautilusBurnDrive       *drive;
 	GtkTreeModel            *model;
-        GtkTreeIter              iter;
+	GtkTreeIter              iter;
+
+	default_speed_index = 0;
 
 	/* Find active recorder: */
 	drive = lookup_current_recorder (xml);
@@ -1689,29 +1755,34 @@ refresh_dialog (GladeXML *xml)
 			    -1);
 
 	if (drive) {
-		int max_speed_write = nautilus_burn_drive_get_max_speed_write (drive);
+		const int *write_speeds;
 
-		for (i = 1; i <= max_speed_write; i++) {
-			name = g_strdup_printf ("%d \303\227", i);
+		write_speeds = nautilus_burn_drive_get_write_speeds (drive);
 
+		for (i = 0; write_speeds[i] > 0; i++) {
+
+			if (write_speeds[i] == default_speed) {
+				default_speed_index = i + 1;
+			}
+
+			name = g_strdup_printf ("%d \303\227", write_speeds[i]);
 			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    0, name,
-					    1, i,
+					    1, write_speeds [i],
 					    -1);
 			g_free (name);
 		}
 
-		/* Disable speed if max speed < 1 */
-		gtk_widget_set_sensitive (combobox, max_speed_write > 0);
-
+		/* Disable speed if no items in list */
+		gtk_widget_set_sensitive (combobox, i > 0);
 	}
 
-	/* for now assume equivalence between index in comboxbox and speed */
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), default_speed);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), default_speed_index);
 
-	if (drive)
+	if (drive) {
 		nautilus_burn_drive_unref (drive);
+	}
 }
 
 static void
@@ -2160,7 +2231,7 @@ selection_dialog_response (GtkWidget                *dialog,
 
 		/* eject all discs */
 		/* FIXME: make this gconf configurable? */
-		eject = TRUE;
+		eject = FALSE;
 
 		speed = get_speed_selection (speed_combobox);
 
@@ -2218,8 +2289,8 @@ fill_selection_dialog (NautilusBurnerSourceData *burn_data)
 
 			goto done;
 		}
-
 		nautilus_burn_drive_unmount (burn_data->source_drive);
+		disc_name = nautilus_burn_drive_get_media_label (burn_data->source_drive);
 		burn_data->size = nautilus_burn_drive_get_media_size (burn_data->source_drive);
 
 		if (burn_data->size <= 0) {
@@ -2266,6 +2337,8 @@ fill_selection_dialog (NautilusBurnerSourceData *burn_data)
 	return FALSE;
 
  done:
+	g_free (disc_name);
+
 	gtk_main_quit ();
 
 	return FALSE;
@@ -2338,6 +2411,28 @@ create_selection_dialog (NautilusBurnerSourceData *burn_data)
 	return FALSE;
 }
 
+static char *
+expand_path_input (const char *input)
+{
+	char *uri;
+	char *path;
+
+	uri = gnome_vfs_make_uri_from_input (input);
+	if (uri == NULL) {
+		g_warning ("Could not parse filename");
+		return NULL;
+	}
+
+	path = gnome_vfs_get_local_path_from_uri (uri);
+	g_free (uri);
+
+	if (path == NULL) {
+		g_warning ("Only local files are supported at this time");
+	}
+
+	return path;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -2384,14 +2479,34 @@ main (int argc, char *argv[])
 	}
 
 	burn_data = g_new0 (NautilusBurnerSourceData, 1);
-	burn_data->source_cue = g_strdup (source_cue);
-	burn_data->source_iso = g_strdup (source_iso);
+
+	if (source_iso != NULL) {
+		char *path;
+
+		path = expand_path_input (source_iso);
+		if (path == NULL) {
+			goto out;
+		}
+		burn_data->source_iso = path;
+	}
+
+	if (source_cue != NULL) {
+		char *path;
+
+		path = expand_path_input (source_cue);
+		if (path == NULL) {
+			goto out;
+		}
+		burn_data->source_cue = path;
+	}
+
 	burn_data->source_device = g_strdup (source_device);
 
 	g_idle_add ((GSourceFunc)create_selection_dialog, burn_data);
 
 	gtk_main ();
 
+ out:
 	g_free (burn_data->source_cue);
 	g_free (burn_data->source_iso);
 	g_free (burn_data->source_device);
